

function CacheConfig(conf){
	const MB = 1024*1024;
	const DAYS = 1000*60*60*24;
	const MINUTES = 1000*60;
	const MAX_CACHE_SIZE = 200* MB;
	const MAX_AGE = 60 * MINUTES;

	var cacheSize = 0;
	var config = conf;
	var keys = new Array();
	var cache = new Object();
	this.cache = cache;
	
	var cacheFile; 
	
	this.getCacheSize  = function() {
		return cacheSize;
	}
	
	function init(){
		cacheFile = config.getBaseDir();
		cacheFile.append("cache.xml");
		if(!cacheFile.exists()){
			cacheFile.create(Components.interfaces.nsIFile.NORMAL_FILE_TYPE, 0644);
			save();
		}else{
			load();
		}
	}
	
	this.clearOldKeys = function(newKeys){
		dump("Clearing old keys \n");
		function isOldKey(el, idx, array){
			if(newKeys.indexOf(el)==-1){
				return true;
			}else{
				return false;
			}
		}
		var oldKeys = keys.filter(isOldKey);
		dump(oldKeys.length +" oldkeys found \n");
		function clear(key){
			dump("Clearing "  + key +"\n");
			keys.splice(keys.indexOf(key), 1);
			delete cache[key];
			save();
		}
		oldKeys.forEach(clear);
	}
	
	function add(key, filename, fileSize){
		//dump("Cachesize: " + cacheSize + " filesize: " + fileSize + "\n");
		cacheSize = parseInt(cacheSize) + parseInt(fileSize);
		keys.push(key);
		cache[key] = new CacheObject();
		cache[key].key = key;
		cache[key].basedir = getCacheDir();
		cache[key].filename = filename;
		cache[key].fileSize =  fileSize;
		cleanCache();
		save();
	}
	
	function cleanCache(){
		dump("Cleaning cache \n");
		var eldestKeyIdx = findEldest();
		if(eldestKeyIdx == -1){ dump("Cache Empty\n"); 		return; };
		var eldestKey = keys[eldestKeyIdx];
		var eldest = cache[eldestKey];
		//dump("EldestKey: " + eldestKey + " Eldest: " + eldest);
		if(eldest){  
			dump("Cachesize: " + cacheSize + " (" + MAX_CACHE_SIZE + ")\n");
			dump("Eldest: "+eldestKey + " age: "+(Date.now() - eldest.timestamp ) + " (" + MAX_AGE+")\n");
			if(cacheSize > MAX_CACHE_SIZE  || (Date.now() - eldest.timestamp) > MAX_AGE){
				dump("Removing " + eldestKey);
				remove(eldestKey);
			}
			}else{
				//No need to remove anything.
			}
	}
	
	this.cleanCache = cleanCache;
	
	function remove(key){
		dump("Removing " + key +" from " + cache[key].url + "\n");
		/*for(var i=0; i<keys.length; i++){
			if(keys[i] == key){
				keys.splice(i, 1);
			}
		}*/
		var cacheObject = cache[key];
		var file = getCacheDir();
		file.append(cacheObject.filename);
		if(file.exists()){
			cacheSize -= file.fileSize;
			file.remove(false);
		}
		cache[key].deleted = true;;		
		save();
		cleanCache();
	}
	
	
	function findEldest(){
		dump("keys.length " + keys.length + "\n");
		if(keys.length>0){
			var eldest=-1;
			var eldestTime = -1;
			for(var i=0; i<keys.length; i++){
				if((!cache[keys[i]].deleted)){
					if((eldestTime==-1 || keys[i].timestamp < eldestTime)){
						eldest =i;
						eldestTime = keys[i].timestamp;
					}
				}else{
					//dump(keys[i] + " deleted  " +cache[keys[i]].deleted + ".  Skipping!\n");
				}
			}
			return eldest;
		}
	}
	
	this.getObject = function(key){
		if(this.cache[key]){ return this.cache[key]; }
	}
	
	
	this.add = add;
	
	function save(){
		var document = Components.classes["@mozilla.org/xul/xul-document;1"].createInstance(Components.interfaces.nsIDOMDocument);
		var doc = document.implementation.createDocument("", "CacheConfig", null);
		
		for(var i=0; i<keys.length; i++){
			var key = keys[i];
			var video = doc.createElement("video");
			video.setAttribute("id", key);
			if(cache[key].deleted){
				video.setAttribute("deleted", "true");
			}else{
				video.setAttribute("localUrl", cache[key].filename);
				video.setAttribute("size", parseInt(cache[key].filesize));
				video.setAttribute("timestamp", parseInt(cache[key].timestamp));
			}
			doc.firstChild.appendChild(video);
		}
		
		
		var stream = Components.classes["@mozilla.org/network/file-output-stream;1"].createInstance(Components.interfaces.nsIFileOutputStream);
		stream.init(cacheFile, 0x02 | 0x08 | 0x20, 420, 0);
		var xmlserializer = Components.classes["@mozilla.org/xmlextras/xmlserializer;1"].createInstance(Components.interfaces.nsIDOMSerializer);
		xmlserializer.serializeToStream(doc, stream, "utf-8");
		stream.close();
   
	};
	
	function getCacheDir(){
		var file = config.getBaseDir();
		file.append("cache");
		if(!file.exists()){
			file.create(Components.interfaces.nsIFile.DIRECTORY_TYPE, 0755);
		}
		return file;
	}
	
	this.getCacheDir = getCacheDir;
	
	this.clear = function(){
	}
	
	function load(){
		var domParser = Components.classes["@mozilla.org/xmlextras/domparser;1"].createInstance(Components.interfaces.nsIDOMParser);
		var uri = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService).newFileURI(cacheFile);
		var configContents = getContents(uri.spec);
		var doc = domParser.parseFromString(configContents, "text/xml");
		
		var path = "//CacheConfig/video";
		var nodes = doc.evaluate(path, doc, null, Components.interfaces.nsIDOMXPathResult.ANY_TYPE, null);
		var node = nodes.iterateNext();
		while(node){
			var id = node.getAttribute("id");
			keys.push(id);
			var deleted = node.getAttribute("deleted");
			if(deleted){
				cache[id] = new CacheObject();
				cache[id].key = id;
				cache[id].deleted = true;
			}else{
				cache[id] = new CacheObject();
				cache[id].key = id;
				cache[id].basedir = getCacheDir();
				cache[id].filename = node.getAttribute("localUrl");
				cache[id].filesize = node.getAttribute("size");
				cacheSize = parseInt(cacheSize) + parseInt(cache[id].filesize);
				cache[id].timestamp = node.getAttribute("timestamp");
			}
			node = nodes.iterateNext();
		}
		
		 
	}
	
	init();
}